@@ -1,23 +1,10 @@
 /*
- * Copyright 2009 Chani Armitage <chani@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License version 2 as
- *   published by the Free Software Foundation
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2009 Chani Armitage <chani@kde.org>
 
-#ifndef APPSENGINE_H
-#define APPSENGINE_H
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
+
+#pragma once
 
 // plasma
 #include <Plasma/DataEngine>
@@ -53,13 +40,8 @@ public:
 protected:
     virtual void init();
 
-private Q_SLOTS:
-    void sycocaChanged(const QStringList &changes);
-
 private:
     friend class AppSource;
     void addGroup(KServiceGroup::Ptr group);
     void addApp(KService::Ptr app);
 };
-
-#endif // TASKSENGINE_H
