@@ -285,6 +285,7 @@ void TasksModel::Private::initModels()
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::filterByVirtualDesktopChanged, q, &TasksModel::filterByVirtualDesktopChanged);
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::filterByScreenChanged, q, &TasksModel::filterByScreenChanged);
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::filterByActivityChanged, q, &TasksModel::filterByActivityChanged);
+    QObject::connect(filterProxyModel, &TaskFilterProxyModel::filterMinimizedChanged, q, &TasksModel::filterMinimizedChanged);
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::filterNotMinimizedChanged, q, &TasksModel::filterNotMinimizedChanged);
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::filterNotMaximizedChanged, q, &TasksModel::filterNotMaximizedChanged);
     QObject::connect(filterProxyModel, &TaskFilterProxyModel::filterHiddenChanged, q, &TasksModel::filterHiddenChanged);
@@ -1151,6 +1152,16 @@ void TasksModel::setFilterByActivity(bool filter)
     d->filterProxyModel->setFilterByActivity(filter);
 }
 
+bool TasksModel::filterMinimized() const
+{
+    return d->filterProxyModel->filterMinimized();
+}
+
+void TasksModel::setFilterMinimized(bool filter)
+{
+    d->filterProxyModel->setFilterMinimized(filter);
+}
+
 bool TasksModel::filterNotMinimized() const
 {
     return d->filterProxyModel->filterNotMinimized();
@@ -1375,7 +1386,7 @@ bool TasksModel::taskReorderingEnabled() const
 void TasksModel::setTaskReorderingEnabled(bool enabled)
 {
     enabled ? setDynamicSortFilter(true) : setDynamicSortFilter(false);
-    
+
     Q_EMIT taskReorderingEnabledChanged();
 }
 
@@ -1592,9 +1603,10 @@ void TasksModel::requestActivities(const QModelIndex &index, const QStringList &
 
 void TasksModel::requestPublishDelegateGeometry(const QModelIndex &index, const QRect &geometry, QObject *delegate)
 {
-    if (index.isValid() && index.model() == this) {
-        d->abstractTasksSourceModel->requestPublishDelegateGeometry(mapToSource(index), geometry, delegate);
+    if (!index.isValid() || index.model() != this || !index.data(AbstractTasksModel::IsWindow).toBool()) {
+        return;
     }
+    d->abstractTasksSourceModel->requestPublishDelegateGeometry(mapToSource(index), geometry, delegate);
 }
 
 void TasksModel::requestToggleGrouping(const QModelIndex &index)
@@ -1607,6 +1619,13 @@ void TasksModel::requestToggleGrouping(const QModelIndex &index)
 
 bool TasksModel::move(int row, int newPos, const QModelIndex &parent)
 {
+    /*
+     * NOTE After doing any modification in TasksModel::move, make sure fixes listed below are not regressed.
+     * - https://bugs.kde.org/444816
+     * - https://bugs.kde.org/448912
+     * - https://invent.kde.org/plasma/plasma-workspace/-/commit/ea51795e8c571513e1ff583350ab8649bc857fc2
+     */
+
     if (d->sortMode != SortManual || row == newPos || newPos < 0 || newPos >= rowCount(parent)) {
         return false;
     }
@@ -1687,11 +1706,12 @@ bool TasksModel::move(int row, int newPos, const QModelIndex &parent)
         // Update sort mappings.
         d->sortedPreFilterRows.move(row, newPos);
 
+        endMoveRows();
+
         if (groupingRowIndexParent.isValid()) {
             d->consolidateManualSortMapForGroup(groupingRowIndexParent);
         }
 
-        endMoveRows();
     } else {
         beginMoveRows(parent, row, row, parent, (newPos > row) ? newPos + 1 : newPos);
 
@@ -1715,20 +1735,27 @@ bool TasksModel::move(int row, int newPos, const QModelIndex &parent)
         // Update sort mapping.
         d->sortedPreFilterRows.move(row, newPos);
 
+        endMoveRows();
+
         // If we moved a group parent, consolidate sort map for children.
         if (groupNotDisabled) {
             if (d->groupingProxyModel->rowCount(groupingRowIndex)) {
                 d->consolidateManualSortMapForGroup(groupingRowIndex);
             }
             // Special case: Before moving, the task at newPos is a group parent
-            // Before moving: [Task] [Group parent]
-            // After moving: [Group parent (not consolidated yet)] [Task]
-            if (adjacentGroupingRowIndex.isValid() && d->groupingProxyModel->rowCount(adjacentGroupingRowIndex)) {
+            // Before moving: [Task] [Group parent] [Other task in group]
+            // After moving: [Group parent (not consolidated yet)] [Task, newPos] [Other task in group]
+            if (int childCount = d->groupingProxyModel->rowCount(adjacentGroupingRowIndex); childCount && adjacentGroupingRowIndex.isValid()) {
                 d->consolidateManualSortMapForGroup(adjacentGroupingRowIndex);
+                if (newPos > row) {
+                    newPos += childCount - 1;
+                    // After consolidation: [Group parent (not consolidated yet)] [Other task in group] [Task, newPos]
+                }
+                // No need to consider newPos < row
+                // Before moving: [Group parent, newPos] [Other task in group] [Task]
+                // After moving: [Task, newPos] [Group parent] [Other task in group]
             }
         }
-
-        endMoveRows();
     }
 
     // Resort.
@@ -1749,29 +1776,22 @@ bool TasksModel::move(int row, int newPos, const QModelIndex &parent)
                 const int sortIndex = d->sortedPreFilterRows.indexOf(d->concatProxyModel->mapFromSource(launcherIndex).row());
                 d->sortedPreFilterRows.move(sortIndex, newPos);
 
-                /*
-                 * Before moving:
-                 * [pinned 2 (launcher)] [pinned 2 (window)] [pinned 1 (launcher)] [pinned 1 (window)]
-                 * After moving [pinned 1], sortedPreFilterRows may become:
-                 *  - row > newPos: [pinned 2 (launcher)] [pinned 1 (launcher)] [pinned 1 (window)] [pinned 2 (window)]
-                 *  - row < newPos: [pinned 2 (window)] [pinned 1 (window)] [pinned 1 (launcher)] [pinned 2 (launcher)]
-                 * We need to move [pinned 2 (launcher)] to the left of [pinned 2 (window)]
-                 */
-                if (row > newPos && newPos - 1 >= 0 && newPos + 2 < d->sortedPreFilterRows.size()) {
-                    const QModelIndex beforeIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(newPos - 1), 0); // [pinned 2 (launcher)]
-                    const QModelIndex afterIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(newPos + 2), 0); // [pinned 2 (window)]
+                if (row > newPos && newPos >= 1) {
+                    const QModelIndex beforeIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(newPos - 1), 0);
+                    if (beforeIdx.data(AbstractTasksModel::IsLauncher).toBool()) {
+                        // Search forward to skip grouped tasks
+                        int afterPos = newPos + 1;
+                        for (; afterPos < d->sortedPreFilterRows.size(); ++afterPos) {
+                            const QModelIndex tempIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(afterPos), 0);
+                            if (!appsMatch(idx, tempIdx)) {
+                                break;
+                            }
+                        }
 
-                    if (appsMatch(beforeIdx, afterIdx)) {
-                        // Move [pinned 2 (launcher)] before [pinned 2 (window)]
-                        d->sortedPreFilterRows.move(newPos - 1, newPos + 2);
-                    }
-                } else if (row < newPos && newPos - 2 >= 0 && newPos + 1 < d->sortedPreFilterRows.size()) {
-                    const QModelIndex beforeIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(newPos - 2), 0); // [pinned 2 (window)]
-                    const QModelIndex afterIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(newPos + 1), 0); // [pinned 2 (launcher)]
-
-                    if (appsMatch(beforeIdx, afterIdx)) {
-                        // Move [pinned 2 (launcher)] before [pinned 2 (window)]
-                        d->sortedPreFilterRows.move(newPos + 1, newPos - 2);
+                        const QModelIndex afterIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(afterPos), 0);
+                        if (appsMatch(beforeIdx, afterIdx)) {
+                            d->sortedPreFilterRows.move(newPos - 1, afterPos - 1);
+                        }
                     }
                 }
                 // Otherwise move matching windows to after the launcher task (they are
@@ -1799,8 +1819,10 @@ bool TasksModel::move(int row, int newPos, const QModelIndex &parent)
              */
             const QModelIndex beforeIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(newPos - 1), 0, parent);
             const QModelIndex afterIdx = d->concatProxyModel->index(d->sortedPreFilterRows.at(newPos + 1), 0, parent);
+            // BUG 462508: check if any item is a launcher
+            const bool hasLauncher = beforeIdx.data(AbstractTasksModel::IsLauncher).toBool() || afterIdx.data(AbstractTasksModel::IsLauncher).toBool();
 
-            if (appsMatch(beforeIdx, afterIdx)) {
+            if (hasLauncher && appsMatch(beforeIdx, afterIdx)) {
                 // after adjusting: [unpinned] [pinned 1 (launcher item)] [pinned 1]
                 d->sortedPreFilterRows.move(newPos, newPos + (row < newPos ? 1 : -1));
             }
