@@ -67,42 +67,65 @@ AppData appDataFromUrl(const QUrl &url, const QIcon &fallbackIcon)
         }
     }
 
-    if (url.isLocalFile() && KDesktopFile::isDesktopFile(url.toLocalFile())) {
-        const KService::Ptr service = KService::serviceByStorageId(url.fileName());
+    if (url.isLocalFile()) {
+        if (KDesktopFile::isDesktopFile(url.toLocalFile())) {
+            const KService::Ptr service = KService::serviceByStorageId(url.fileName());
 
-        // Resolve to non-absolute menuId-based URL if possible.
-        if (service) {
-            const QString &menuId = service->menuId();
+            // Resolve to non-absolute menuId-based URL if possible.
+            if (service) {
+                const QString &menuId = service->menuId();
 
-            if (!menuId.isEmpty()) {
-                data.url = QUrl(QLatin1String("applications:") + menuId);
-            }
-        }
-
-        if (service && QUrl::fromLocalFile(service->entryPath()) == url) {
-            data.name = service->name();
-            data.genericName = service->genericName();
-            data.id = service->storageId();
-
-            if (data.icon.isNull()) {
-                data.icon = QIcon::fromTheme(service->icon());
-            }
-        } else {
-            KDesktopFile f(url.toLocalFile());
-            if (f.tryExec()) {
-                data.name = f.readName();
-                data.genericName = f.readGenericName();
-                data.id = QUrl::fromLocalFile(f.fileName()).fileName();
-
-                if (data.icon.isNull()) {
-                    data.icon = QIcon::fromTheme(f.readIcon());
+                if (!menuId.isEmpty()) {
+                    data.url = QUrl(QLatin1String("applications:") + menuId);
                 }
             }
+
+            if (service && QUrl::fromLocalFile(service->entryPath()) == url) {
+                data.name = service->name();
+                data.genericName = service->genericName();
+                data.id = service->storageId();
+
+                if (data.icon.isNull()) {
+                    data.icon = QIcon::fromTheme(service->icon());
+                }
+            } else {
+                KDesktopFile f(url.toLocalFile());
+                if (f.tryExec()) {
+                    data.name = f.readName();
+                    data.genericName = f.readGenericName();
+                    data.id = QUrl::fromLocalFile(f.fileName()).fileName();
+
+                    if (data.icon.isNull()) {
+                        const QString iconValue = f.readIcon();
+                        if (QIcon::hasThemeIcon(iconValue)) {
+                            data.icon = QIcon::fromTheme(iconValue);
+                        } else if (!iconValue.startsWith(QDir::separator())) {
+                            const QString iconValueWithoutSuffix = iconValue.left(iconValue.lastIndexOf(QLatin1Char('.')));
+                            // Find an icon in the same folder
+                            const QDir sameDir = QFileInfo(url.toLocalFile()).absoluteDir();
+                            const auto iconList = sameDir.entryInfoList(
+                                {
+                                    QStringLiteral("*.png").arg(iconValueWithoutSuffix),
+                                    QStringLiteral("*.svg").arg(iconValueWithoutSuffix),
+                                },
+                                QDir::Files);
+                            if (!iconList.empty()) {
+                                data.icon = QIcon(iconList[0].absoluteFilePath());
+                            }
+                        } else {
+                            data.icon = QIcon(iconValue);
+                        }
+                    }
+                }
+            }
+
+            if (data.id.endsWith(".desktop")) {
+                data.id = data.id.left(data.id.length() - 8);
+            }
+        } else {
+            data.id = url.fileName();
         }
 
-        if (data.id.endsWith(".desktop")) {
-            data.id = data.id.left(data.id.length() - 8);
-        }
     } else if (url.scheme() == QLatin1String("preferred")) {
         data.id = defaultApplication(url);
 
@@ -456,6 +479,7 @@ KService::List servicesFromPid(quint32 pid, KSharedConfig::Ptr rulesConfig)
     QFile environFile(QStringLiteral("/proc/%1/environ").arg(QString::number(pid)));
     if (environFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
         const QByteArray bamfDesktopFileHint = QByteArrayLiteral("BAMF_DESKTOP_FILE_HINT");
+        const QByteArray appDir = QByteArrayLiteral("APPDIR");
 
         const auto lines = environFile.readAll().split('\0');
         for (const QByteArray &line : lines) {
@@ -473,6 +497,14 @@ KService::List servicesFromPid(quint32 pid, KSharedConfig::Ptr rulesConfig)
                     return {service};
                 }
                 break;
+            } else if (key == appDir) {
+                // For AppImage
+                const QByteArray value = line.mid(equalsIdx + 1);
+                const auto desktopFileList = QDir(QString::fromUtf8(value)).entryInfoList(QStringList{QStringLiteral("*.desktop")}, QDir::Files);
+                if (!desktopFileList.empty()) {
+                    return {QExplicitlySharedDataPointer<KService>(new KService(desktopFileList[0].absoluteFilePath()))};
+                }
+                break;
             }
         }
     }
