@@ -2,6 +2,7 @@
     SPDX-FileCopyrightText: 2006 Aaron Seigo <aseigo@kde.org>
     SPDX-FileCopyrightText: 2014 Vishesh Handa <vhanda@kde.org>
     SPDX-FileCopyrightText: 2016-2020 Harald Sitter <sitter@kde.org>
+    SPDX-FileCopyrightText: 2022 Alexander Lohnau <alexander.lohnau@gmx.de>
 
     SPDX-License-Identifier: LGPL-2.0-only
 */
@@ -24,6 +25,7 @@
 #include <KLocalizedString>
 #include <KNotificationJobUiDelegate>
 #include <KServiceAction>
+#include <KShell>
 #include <KStringHandler>
 #include <KSycoca>
 
@@ -63,25 +65,19 @@ inline bool contains(const QStringList &results, const QStringList &queryList)
 class ServiceFinder
 {
 public:
-    ServiceFinder(ServiceRunner *runner)
+    ServiceFinder(ServiceRunner *runner, const QList<KService::Ptr> &list)
         : m_runner(runner)
+        , m_services(list)
     {
     }
 
     void match(Plasma::RunnerContext &context)
     {
-        if (!context.isValid()) {
-            return;
-        }
-
-        KSycoca::disableAutoRebuild();
-
         term = context.query();
         // Splitting the query term to match using subsequences
         queryList = term.split(QLatin1Char(' '));
         weightedTermLength = weightedLength(term);
 
-        matchExectuables();
         matchNameKeywordAndGenericName();
         matchCategories();
         matchJumpListActions();
@@ -119,16 +115,6 @@ private:
         return ret;
     }
 
-    bool shouldParseExec(const KService::Ptr &ptr)
-    {
-        // Do not parse the exec of Progressive Web Apps, see BUG 460796
-        const QLatin1String chromiumPwaWindowClassPrefix("crx_");
-        // Also ignore Exec lines of flatpaks, those contain RDN identifiers that would cause false positives, BUG: 461241
-        const QLatin1String flatpakAppEntryPath("/flatpak/exports/share/");
-        return !ptr->entryPath().contains(flatpakAppEntryPath)
-            && !ptr->property(QStringLiteral("StartupWMClass")).toString().startsWith(chromiumPwaWindowClassPrefix);
-    }
-
     qreal increaseMatchRelavance(const KService::Ptr &service, const QStringList &strList, const QString &category)
     {
         // Increment the relevance based on all the words (other than the first) of the query list
@@ -176,12 +162,36 @@ private:
         match.setUrls({QUrl::fromLocalFile(path)});
 
         QString exec = service->exec();
-        // We have a snap, remove the ENV variable
-        if (exec.contains(QLatin1String("BAMF_DESKTOP_FILE_HINT"))) {
-            const static QRegularExpression snapCleanupRegex(QStringLiteral("env BAMF_DESKTOP_FILE_HINT=.+ "));
-            exec.remove(snapCleanupRegex);
+
+        QStringList resultingArgs = KIO::DesktopExecParser(KService(QString(), exec, QString()), {}).resultingArguments();
+
+        // Remove any environment variables.
+        if (KIO::DesktopExecParser::executableName(exec) == QLatin1String("env")) {
+            resultingArgs.removeFirst(); // remove "env".
+
+            while (!resultingArgs.isEmpty() && resultingArgs.first().contains(QLatin1Char('='))) {
+                resultingArgs.removeFirst();
+            }
+
+            // Now parse it again to resolve the path.
+            resultingArgs = KIO::DesktopExecParser(KService(QString(), KShell::joinArgs(resultingArgs), QString()), {}).resultingArguments();
         }
-        const QStringList resultingArgs = KIO::DesktopExecParser(KService(QString(), exec, QString()), {}).resultingArguments();
+
+        // Remove special arguments that have no real impact on the application.
+        static const auto specialArgs = {QStringLiteral("-qwindowtitle"), QStringLiteral("-qwindowicon"), QStringLiteral("--started-from-file")};
+
+        for (const auto &specialArg : specialArgs) {
+            int index = resultingArgs.indexOf(specialArg);
+            if (index > -1) {
+                if (resultingArgs.count() > index) {
+                    resultingArgs.removeAt(index);
+                }
+                if (resultingArgs.count() > index) {
+                    resultingArgs.removeAt(index); // remove value, too, if any.
+                }
+            }
+        }
+
         match.setId(QStringLiteral("exec://") + resultingArgs.join(QLatin1Char(' ')));
         if (!service->genericName().isEmpty() && service->genericName() != name) {
             match.setSubtext(service->genericName());
@@ -194,39 +204,11 @@ private:
         }
     }
 
-    void matchExectuables()
-    {
-        if (weightedTermLength < 2) {
-            return;
-        }
-
-        const auto executablesFilter = [this](const KService::Ptr &service) {
-            return QString::compare(service->name(), term, Qt::CaseInsensitive) == 0;
-        };
-        const KService::List services = KApplicationTrader::query(executablesFilter);
-
-        if (services.isEmpty()) {
-            return;
-        }
-
-        for (const KService::Ptr &service : services) {
-            qCDebug(RUNNER_SERVICES) << service->name() << "is an exact match!" << service->storageId() << service->exec();
-            if (disqualify(service)) {
-                continue;
-            }
-            Plasma::QueryMatch match(m_runner);
-            match.setType(Plasma::QueryMatch::ExactMatch);
-            setupMatch(service, match);
-            match.setRelevance(1);
-            matches << match;
-        }
-    }
-
     void matchNameKeywordAndGenericName()
     {
         const auto nameKeywordAndGenericNameFilter = [this](const KService::Ptr &service) {
             // Name
-            if (contains(service->name(), queryList) || (weightedTermLength >= 3 && shouldParseExec(service) && contains(service->exec(), queryList))) {
+            if (contains(service->name(), queryList)) {
                 return true;
             }
             // If the term length is < 3, no real point searching the Keywords and GenericName
@@ -249,11 +231,8 @@ private:
             return false;
         };
 
-        const KService::List services = KApplicationTrader::query(nameKeywordAndGenericNameFilter);
-
-        qCDebug(RUNNER_SERVICES) << "got " << services.count() << " services from " << query;
-        for (const KService::Ptr &service : services) {
-            if (disqualify(service)) {
+        for (const KService::Ptr &service : m_services) {
+            if (!nameKeywordAndGenericNameFilter(service) || disqualify(service)) {
                 continue;
             }
 
@@ -291,13 +270,6 @@ private:
                 if (service->genericName().startsWith(queryList[0], Qt::CaseInsensitive)) {
                     relevance += 0.05;
                 }
-            } else if (exec.contains(queryList[0], Qt::CaseInsensitive)) {
-                relevance = 0.7;
-                relevance += increaseMatchRelavance(service, queryList, QStringLiteral("Exec"));
-
-                if (exec.startsWith(queryList[0], Qt::CaseInsensitive)) {
-                    relevance += 0.05;
-                }
             } else if (service->comment().contains(queryList[0], Qt::CaseInsensitive)) {
                 relevance = 0.5;
                 relevance += increaseMatchRelavance(service, queryList, QStringLiteral("Comment"));
@@ -324,14 +296,12 @@ private:
         const auto categoriesFilter = [this](const KService::Ptr &service) {
             return contains(service->categories(), queryList);
         };
-        // search for applications whose categories contains the query
-        const auto services = KApplicationTrader::query(categoriesFilter);
 
-        for (const KService::Ptr &service : services) {
-            qCDebug(RUNNER_SERVICES) << service->name() << "is an exact match!" << service->storageId() << service->exec();
-            if (disqualify(service)) {
+        for (const KService::Ptr &service : m_services) {
+            if (!categoriesFilter(service) || disqualify(service)) {
                 continue;
             }
+            qCDebug(RUNNER_SERVICES) << service->name() << "is an exact match!" << service->storageId() << service->exec();
 
             Plasma::QueryMatch match(m_runner);
             match.setType(Plasma::QueryMatch::PossibleMatch);
@@ -365,10 +335,8 @@ private:
         const auto hasActionsFilter = [](const KService::Ptr &service) {
             return !service->actions().isEmpty();
         };
-        const auto services = KApplicationTrader::query(hasActionsFilter);
-
-        for (const KService::Ptr &service : services) {
-            if (service->noDisplay()) {
+        for (const KService::Ptr &service : m_services) {
+            if (!hasActionsFilter(service) || service->noDisplay()) {
                 continue;
             }
 
@@ -427,6 +395,7 @@ private:
 
     ServiceRunner *m_runner;
     QSet<QString> m_seen;
+    const QList<KService::Ptr> m_services;
 
     QList<Plasma::QueryMatch> matches;
     QString query;
@@ -448,9 +417,10 @@ ServiceRunner::~ServiceRunner() = default;
 
 void ServiceRunner::match(Plasma::RunnerContext &context)
 {
-    // This helper class aids in keeping state across numerous
-    // different queries that together form the matches set.
-    ServiceFinder finder(this);
+    KSycoca::disableAutoRebuild();
+    ServiceFinder finder(this, KApplicationTrader::query([](const KService::Ptr &) {
+                             return true;
+                         }));
     finder.match(context);
 }
 
